==> db-4.0.14.tar.gz: db-4.0.14/os/os_clock.c <==
/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 2001
 *	Sleepycat Software.  All rights reserved.
 */

#include "db_config.h"

#ifndef lint
static const char revid[] = "$Id: os_clock.c,v 1.6 2001/09/07 18:17:49 krinsky Exp $";
#endif /* not lint */

#ifndef NO_SYSTEM_INCLUDES
#include <sys/types.h>

#if TIME_WITH_SYS_TIME
#include <sys/time.h>
#include <time.h>
#else
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif /* HAVE_SYS_TIME_H */
#endif /* TIME_WITH SYS_TIME */

#include <string.h>
#endif

#include "db_int.h"
#include "os_jump.h"

/*
 * __os_clock --
 *	Return the current time-of-day clock in seconds and microseconds.
 *
 * PUBLIC: int __os_clock __P((DB_ENV *, u_int32_t *, u_int32_t *));
 */
int
__os_clock(dbenv, secsp, usecsp)
	DB_ENV *dbenv;
	u_int32_t *secsp, *usecsp;	/* Seconds and microseconds. */
{
#if defined(HAVE_GETTIMEOFDAY)
	struct timeval tp;
	int ret;

retry:	if (gettimeofday(&tp, NULL) != 0) {
		if ((ret = __os_get_errno()) == EINTR)
			goto retry;
		__db_err(dbenv, "gettimeofday: %s", strerror(ret));
		return (ret);
	}

	if (secsp != NULL)
		*secsp = tp.tv_sec;
	if (usecsp != NULL)
		*usecsp = tp.tv_usec;
#endif
#if !defined(HAVE_GETTIMEOFDAY) && defined(HAVE_CLOCK_GETTIME)
	struct timespec tp;
	int ret;

retry:	if (clock_gettime(CLOCK_REALTIME, &tp) != 0) {
		if ((ret = __os_get_errno()) == EINTR)
			goto retry;
		__db_err(dbenv, "clock_gettime: %s", strerror(ret));
		return (ret);
	}

	if (secsp != NULL)
		*secsp = tp.tv_sec;
	if (usecsp != NULL)
		*usecsp = tp.tv_nsec / 1000;
#endif
#if !defined(HAVE_GETTIMEOFDAY) && !defined(HAVE_CLOCK_GETTIME)
	time_t now;
	int ret;

	if (time(&now) == (time_t)-1) {
		ret = __os_get_errno();
		__db_err(dbenv, "time: %s", strerror(ret));
		return (ret);
	}

	if (secsp != NULL)
		*secsp = now;
	if (usecsp != NULL)
		*usecsp = 0;
#endif
	return (0);
}

==> db-4.3.28.tar.gz: db-4.3.28/os/os_clock.c <==
/*-
 * See the file LICENSE for redistribution information.
 *
 * Copyright (c) 2001-2004
 *	Sleepycat Software.  All rights reserved.
 *
 * $Id: os_clock.c,v 1.15 2004/07/06 17:33:14 bostic Exp $
 */

#include "db_config.h"

#ifndef NO_SYSTEM_INCLUDES
#include <sys/types.h>

#if TIME_WITH_SYS_TIME
#include <sys/time.h>
#include <time.h>
#else
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif /* HAVE_SYS_TIME_H */
#endif /* TIME_WITH SYS_TIME */

#include <string.h>
#endif

#include "db_int.h"

/*
 * __os_clock --
 *	Return the current time-of-day clock in seconds and microseconds.
 *
 * PUBLIC: void __os_clock __P((DB_ENV *, u_int32_t *, u_int32_t *));
 */
void
__os_clock(dbenv, secsp, usecsp)
	DB_ENV *dbenv;
	u_int32_t *secsp, *usecsp;	/* Seconds and microseconds. */
{
	const char *sc;
	int ret;

#if defined(HAVE_GETTIMEOFDAY)
	struct timeval tp;

	RETRY_CHK((gettimeofday(&tp, NULL)), ret);
	if (ret != 0) {
		sc = "gettimeofday";
		goto err;
	}

	if (secsp != NULL)
		*secsp = (u_int32_t)tp.tv_sec;
	if (usecsp != NULL)
		*usecsp = (u_int32_t)tp.tv_usec;
#endif
#if !defined(HAVE_GETTIMEOFDAY) && defined(HAVE_CLOCK_GETTIME)
	struct timespec tp;

	RETRY_CHK((clock_gettime(CLOCK_REALTIME, &tp)), ret);
	if (ret != 0) {
		sc = "clock_gettime";
		goto err;
	}

	if (secsp != NULL)
		*secsp = tp.tv_sec;
	if (usecsp != NULL)
		*usecsp = tp.tv_nsec / 1000;
#endif
#if !defined(HAVE_GETTIMEOFDAY) && !defined(HAVE_CLOCK_GETTIME)
	time_t now;

	RETRY_CHK((time(&now) == (time_t)-1 ? 1 : 0), ret);
	if (ret != 0) {
		sc = "time";
		goto err;
	}

	if (secsp != NULL)
		*secsp = now;
	if (usecsp != NULL)
		*usecsp = 0;
#endif
	return;

err:	__db_err(dbenv, "%s: %s", sc, strerror(ret));
	(void)__db_panic(dbenv, ret);
}
